@@ -7,6 +7,8 @@ use Catalyst::Utils;
 use Tree::Simple qw/use_weak_refs/;
 use Tree::Simple::Visitor::FindByUID;
 
+use namespace::clean -except => 'meta';
+
 has enable => (is => 'rw', required => 1, default => sub{ 1 });
 has tree => (
              is => 'ro',
@@ -82,6 +84,10 @@ sub profile {
     return $node->getUID;
 }
 
+sub created {
+    return @{ shift->{tree}->getNodeValue->{t} };
+}
+
 sub elapsed {
     return tv_interval(shift->{tree}->getNodeValue->{t});
 }
@@ -119,7 +125,7 @@ sub _get_uid {
     $visitor->searchForUID($uid);
     $self->accept($visitor);
     return $visitor->getResult;
-} 
+}
 
 sub addChild {
     my $self = shift;
@@ -154,13 +160,14 @@ sub getNodeValue {
     $self->tree->getNodeValue( @_ )->{ t };
 }
 
-no Moose;
 __PACKAGE__->meta->make_immutable();
 
 1;
 
 __END__
 
+=for stopwords addChild getNodeValue mysub rollup setNodeValue
+
 =head1 NAME
 
 Catalyst::Stats - Catalyst Timing Statistics Class
@@ -205,7 +212,7 @@ be like this:
     $c->stats->profile("completed second part of critical bit");
     # more code
     ...
-    $c->stats->profile(end => "mysub"); 
+    $c->stats->profile(end => "mysub");
   }
 
 Supposing mysub was called from the action "process" inside a Catalyst
@@ -232,7 +239,7 @@ part 0.111s.
 
 =head2 new
 
-Constructor. 
+Constructor.
 
     $stats = Catalyst::Stats->new;
 
@@ -251,7 +258,7 @@ Enable or disable stats collection.  By default, stats are enabled after object
 
 Marks a profiling point.  These can appear in pairs, to time the block of code
 between the begin/end pairs, or by themselves, in which case the time of
-execution to the previous profiling point will be reported.  
+execution to the previous profiling point will be reported.
 
 The argument may be either a single comment string or a list of name-value
 pairs.  Thus the following are equivalent:
@@ -296,6 +303,13 @@ The profiling point will be ignored if the UID has not been previously defined.
 Returns the UID of the current point in the profile tree.  The UID is
 automatically assigned if not explicitly given.
 
+=head2 created
+
+    ($seconds, $microseconds) = $stats->created;
+
+Returns the time the object was created, in C<gettimeofday> format, with
+Unix epoch seconds followed by microseconds.
+
 =head2 elapsed
 
     $elapsed = $stats->elapsed
@@ -349,7 +363,7 @@ Catalyst Contributors, see Catalyst.pm
 
 =head1 COPYRIGHT
 
-This program is free software, you can redistribute it and/or modify
+This library is free software. You can redistribute it and/or modify
 it under the same terms as Perl itself.
 
 =cut
